"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const spawn = require("cross-spawn");
const fs = require("fs-extra");
const inquirer = require("inquirer");
const path = require("path");
const cwd = process.cwd();
class Create {
    constructor(projectName, options = {}) {
        this.getInfo = (projectName) => {
            this.inCurrent = projectName === '.';
            this.name = this.inCurrent ? path.relative('../', cwd) : projectName;
            this.targetDir = path.resolve(cwd, projectName || '.');
            this.getTemplatePath();
        };
        this.getTemplatePath = () => {
            const dir = path.join(__dirname, '../../../../');
            if (this.options.template) {
                this.templatePath = this.options.template;
            }
            else {
                if (this.options.spa) {
                    this.templatePath = path.join(dir, 'template/spa');
                }
                else {
                    this.templatePath = path.join(dir, 'template/default');
                }
                if (this.options.plugin) {
                    this.templatePath = path.join(dir, 'template/plugin');
                }
            }
            if (!fs.existsSync(this.templatePath)) {
                console.error(`\n ${chalk_1.default.red('no template')} \n`);
                process.exit(1);
            }
        };
        this.create = () => __awaiter(this, void 0, void 0, function* () {
            const { targetDir } = this;
            console.log(`\n Creating a new React app in ${chalk_1.default.green(targetDir)}. \n`);
            yield this.ensureDir();
        });
        this.ensureDir = () => __awaiter(this, void 0, void 0, function* () {
            const { targetDir, templatePath, name, inCurrent } = this;
            if (fs.existsSync(targetDir)) {
                console.log(` Uh oh! Looks like there's already a directory called ${chalk_1.default.red(name)}.`);
                if (inCurrent) {
                    const { ok } = yield inquirer.prompt({
                        type: 'confirm',
                        name: 'ok',
                        message: 'Generate project in current directory?',
                    });
                    if (!ok) {
                        return false;
                    }
                }
                else {
                    const { ok } = yield inquirer.prompt({
                        type: 'confirm',
                        name: 'ok',
                        message: 'The folder already exists, is it deleted?',
                    });
                    if (!ok) {
                        return false;
                    }
                }
            }
            fs.emptyDirSync(targetDir);
            fs.copySync(templatePath, targetDir);
            this.installModules();
        });
        this.shouldUseYarn = () => {
            try {
                spawn.sync('yarnpkg --version', { stdio: 'ignore' });
                return true;
            }
            catch (e) {
                return false;
            }
        };
        this.installModules = () => {
            const { targetDir, useYarn, name, inCurrent } = this;
            const dependencies = this.getInstallPackage();
            process.chdir(targetDir);
            let command;
            let args;
            if (useYarn) {
                command = 'yarnpkg';
                args = ['add'];
                if (dependencies.length > 0) {
                    args.push('--exact');
                }
                args = [...args, ...dependencies];
                args.push('--cwd');
                args.push(targetDir);
            }
            else {
                command = 'npm';
                args = [
                    'install',
                    '--save',
                    '--save-exact',
                    '--loglevel',
                    'error',
                ].concat(dependencies);
            }
            const child = spawn(command, args, { stdio: 'inherit' });
            console.log(`\n ${chalk_1.default.green('Installing packages.')} This might take a couple of minutes.\n`);
            child.on('close', (code) => {
                if (code !== 0) {
                    console.log(chalk_1.default.red(`error: ${command} ${args.join(' ')}`));
                    return;
                }
                console.log();
                console.log(`Success! Created ${chalk_1.default.green(name)} at ${chalk_1.default.green(targetDir)}`);
                console.log('Inside that directory, you can run several commands:');
                console.log();
                console.log(chalk_1.default.cyan(`  ${useYarn ? 'yarn' : 'npm'} start`));
                console.log('    Starts the development server.');
                console.log();
                console.log(chalk_1.default.cyan(`  ${useYarn ? 'yarn' : 'npm'} ${useYarn ? '' : 'run'} build`));
                console.log('    Bundles the app into static files for production.');
                console.log();
                console.log('We suggest that you begin by typing:');
                console.log();
                if (!inCurrent) {
                    console.log(chalk_1.default.cyan('  cd'), name);
                }
                console.log(`  ${chalk_1.default.cyan(`${useYarn ? 'yarn' : 'npm'} start`)}\n`);
            });
        };
        this.getInstallPackage = () => {
            let allDependencies = [
                '@types/react',
                '@types/react-dom',
                'react-dom',
                'react',
                'radish-server',
                'react-hot-loader',
            ];
            if (!this.options.spa) {
                allDependencies = allDependencies.concat(['@types/express', 'express']);
            }
            else {
                allDependencies.unshift('@types/node');
            }
            if (this.options.plugin) {
                return [];
            }
            return allDependencies;
        };
        this.projectName = projectName;
        this.options = options;
        this.useYarn = this.shouldUseYarn();
        this.getInfo(this.projectName);
    }
}
exports.default = Create;
//# sourceMappingURL=Create.js.map